from dataclasses import asdict
from typing import Optional

from vellum.api_resources.abstract.predict_api_resource import PredictAPIResource
from vellum.api_resources.types import SearchOptions, SearchResults


class Search(PredictAPIResource):
    api_path = "v1/search"

    @classmethod
    def run(  # type: ignore
        cls,
        query: str,
        index_id: Optional[str] = None,
        index_name: Optional[str] = None,
        options: Optional[SearchOptions] = None,
    ) -> SearchResults:
        """Peform a semantic search query against a document index.

        Args:
            actuals: A list of CompletionActual objects, each containing the info needed
                to submit a single actual.
            index_id: The ID of the document index that you'd like to search across. You must provide either this
                or index_name.
            index_name: The name of the document index that you'd like to search across. You must provide either
                this or index_id.
            options: An optional SearchOptions object containing options for the search query.
        Returns:
            The result of the generation request.
        """

        if index_id is None and index_name is None:
            raise ValueError("Must provide either index_id or index_name")
        elif index_id is not None and index_name is not None:
            raise ValueError("Must provide only one of index_id or index_name")

        options = options or SearchOptions()

        raw_result = super().run(
            query=query,
            index_id=index_id,
            index_name=index_name,
            options=asdict(options),
        )

        return SearchResults.from_raw(raw_result)



from dataclasses import asdict
from typing import List, Optional

from vellum.api_resources.abstract.predict_api_resource import PredictAPIResource
from vellum.api_resources.types import GenerateRequest, GenerateResult


class Generate(PredictAPIResource):
    api_path = "v1/generate"

    @classmethod
    def run(  # type: ignore
        cls,
        requests: List[GenerateRequest],
        deployment_id: Optional[str] = None,
        deployment_name: Optional[str] = None,
    ) -> GenerateResult:
        """Run a generation request against Vellum's predict API.

        Args:
            requests: A list of GenerationRequest objects, each containing the info needed
                to make a single prediction.
            deployment_id: The ID of the deployment to use for generating predictions. You must provide either this
                or deployment_name.
            deployment_name: The name of the deployment to use for generating predictions. You must provide either
                this or deployment_id.
        Returns:
            The result of the generation request.
        """

        if deployment_id is None and deployment_name is None:
            raise ValueError("Must provide either deployment_id or deployment_name")
        elif deployment_id is not None and deployment_name is not None:
            raise ValueError("Must provide only one of deployment_id or deployment_name")

        raw_result = super().run(
            requests=[asdict(request) for request in requests],
            deployment_id=deployment_id,
            deployment_name=deployment_name,
        )

        return GenerateResult.from_raw(raw_result)



from abc import ABC, abstractmethod
from typing import Any, Optional

import requests
from requests import Response

import vellum

from .exceptions import ApiResourceError


class APIResource(ABC):
    @property
    @abstractmethod
    def api_base(self) -> str:
        pass

    @classmethod
    def _construct_url(cls, path: str) -> str:
        return f"{cls.api_base}/{path}"

    @classmethod
    def _construct_headers(cls, headers: Optional[dict] = None) -> dict:
        api_key = vellum.api_key
        return {"X-API-KEY": api_key, **(headers or {})}

    @classmethod
    def _make_request(cls, method: str, path: str, headers: Optional[dict] = None, **kwargs: Any) -> Response:
        url = cls._construct_url(path)
        headers = cls._construct_headers(headers)

        response = requests.request(method, url, headers=headers, **kwargs)

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise ApiResourceError.from_http_error(e)

        return response



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

import dacite


class Provider(Enum):
    OPENAI = "OPENAI"
    COHERE = "COHERE"
    UNKNOWN = "UNKNOWN"


class FinishReason(Enum):
    LENGTH = "LENGTH"
    STOP = "STOP"
    UNKNOWN = "UNKNOWN"


@dataclass
class TokenLogProbs:
    token: str
    logprob: Optional[float]
    top_logprobs: Optional[Dict[str, float]] = field(repr=False)
    text_offset: int = field(repr=False)


@dataclass
class LogProbs:
    tokens: List[TokenLogProbs] = field(repr=False)
    likelihood: float


@dataclass
class Completion:
    id: str = field(repr=False)
    external_id: Optional[str] = field(repr=False)
    model_version_id: str = field(repr=False)
    text: str
    finish_reason: FinishReason = field(repr=False)
    logprobs: Optional[LogProbs] = field(repr=False)


@dataclass
class CompletionResultData:
    completions: List[Completion]


@dataclass
class CompletionResultError:
    message: str


@dataclass
class CompletionResult:
    data: Optional[CompletionResultData]
    error: Optional[CompletionResultError]


@dataclass
class GenerateResult:
    # The completions generated by the LLM provider, normalized to Vellum's schema.
    results: List[CompletionResult]

    @classmethod
    def from_raw(cls, raw_result: dict) -> GenerateResult:
        return dacite.from_dict(data_class=cls, data=raw_result, config=dacite.Config(cast=[Provider, FinishReason]))

    @property
    def texts(self) -> List[str]:
        return [
            completion.text
            for result in self.results
            for completion in (result.data.completions if result.data else [])
        ]

    @property
    def text(self) -> str:
        if len(self.texts) != 1:
            raise ValueError(f"Expected exactly one completion, but got {len(self.texts)}")

        return self.texts[0]


@dataclass
class GenerateRequest:
    # A mapping between prompt template variables and the values they should be substituted with
    input_values: Dict[str, Any]
    # Optionally provide an external ID for each prompt. This ID will be returned in the result
    # and can later be used to uniquely identify the generation request that produced the result.
    external_ids: Optional[List[str]] = None


@dataclass
class CompletionActual:
    # The id of the completion request, as originally returned by Vellum when making the Generate call.
    # You must provide either this or external_id.
    id: Optional[str] = None
    # The external id of the completion request, as originally provided by you when making the Generate call.
    # You must provide either this or id.
    external_id: Optional[str] = None
    # The quality of the completion, as a number between 0 and 1. 1 is good, 0 is bad.
    quality: Optional[float] = None
    # The desired output text of the completion. You would typically include the original completion text if also
    # passing quality = 1.0. If you deem quality to be < 1.0, you might include what you wish the
    # generation had produced.
    text: Optional[str] = None
    # Optionally include a timestamp representing when this actual was determined.
    timestamp: Optional[datetime] = None

    def __post_init__(self) -> None:
        if self.id is None and self.external_id is None:
            raise ValueError("Either id or external_id must be provided")

        if self.text is None and self.quality is None:
            raise ValueError("Either text and/or quality must be provided")

        if self.quality is not None and (self.quality < 0 or self.quality > 1):
            raise ValueError("quality must be between 0 and 1")


@dataclass
class SubmitCompletionActualsResult:
    success: bool


@dataclass
class SearchWeights:
    semantic_similarity: float = 0.8
    keywords: float = 0.2

    def __post_init__(self) -> None:
        if sum([self.semantic_similarity, self.keywords]) != 1.0:
            raise ValueError("Weights must sum to 1.0")


@dataclass
class ResultMerging:
    enabled: bool


@dataclass
class SearchFilters:
    external_ids: Optional[List[str]] = None


@dataclass
class SearchOptions:
    limit: Optional[int] = 3
    weights: Optional[SearchWeights] = field(default_factory=SearchWeights)
    result_merging: Optional[ResultMerging] = None
    filters: Optional[SearchFilters] = None


@dataclass
class SearchResultDocument:
    id: str
    label: str
    external_id: Optional[str] = None


@dataclass
class SearchResult:
    document: SearchResultDocument = field(repr=False)
    text: str
    keywords: List[str] = field(repr=False)
    score: float = field(repr=False)


@dataclass
class SearchResults:
    results: List[SearchResult]

    @classmethod
    def from_raw(cls, raw_result: dict) -> SearchResults:
        return dacite.from_dict(data_class=cls, data=raw_result)


@dataclass
class DocumentUploadResult:
    document_id: str

    @classmethod
    def from_raw(cls, raw_result: dict) -> DocumentUploadResult:
        return dacite.from_dict(data_class=cls, data=raw_result)



from abc import abstractmethod
from typing import Any, cast

from vellum.api_resources.abstract.api_resource import APIResource


class PredictAPIResource(APIResource):
    @property
    @abstractmethod
    def api_path(self) -> str:
        pass

    api_base = "https://predict.vellum.ai"

    @classmethod
    def run(cls, **kwargs: Any) -> dict:
        result = cast(dict, cls._make_request("POST", cast(str, cls.api_path), json=kwargs).json())

        return result



from typing import IO, List, Optional

from vellum.api_resources.abstract.documents_api_resource import DocumentsAPIResource
from vellum.api_resources.types import DocumentUploadResult


class UploadDocument(DocumentsAPIResource):
    api_path = "v1/upload-document"

    @classmethod
    def run(  # type: ignore
        cls,
        contents: IO,
        label: str,
        add_to_index_names: Optional[List[str]] = None,
        external_id: Optional[str] = None,
        keywords: Optional[List[str]] = None,
    ) -> DocumentUploadResult:
        """Upload a document to later search across it.

        Args:
            contents: The contents of the file that you'd like to upload
            label: A human-friendly name for the document, typically the file name.
            add_to_index_names: The names of document indexes that you'd like this document to be added to.
            external_id: Optionally include a unique ID from your system to this document later.
                Useful if you want to perform updates later.
            keywords: Optionally include keywords to associate with the document that can be used in hybrid search.
        Returns:
            The result of the document upload request.
        """

        raw_result = super().run(
            contents=contents,
            label=label,
            add_to_index_names=add_to_index_names,
            external_id=external_id,
            keywords=keywords,
        )

        return DocumentUploadResult.from_raw(raw_result)



from abc import abstractmethod
import json
import os
from typing import IO, Any, cast

from vellum.api_resources.abstract.api_resource import APIResource


class DocumentsAPIResource(APIResource):
    @property
    @abstractmethod
    def api_path(self) -> str:
        pass

    api_base = "https://documents.vellum.ai"

    @classmethod
    def run(cls, contents: IO, **kwargs: Any) -> dict:
        result = cast(
            dict,
            cls._make_request(
                "POST",
                cast(str, cls.api_path),
                files={"contents": (contents.name, contents, "application/octet-stream")},
                data=kwargs,
            ).json(),
        )

        return result



from __future__ import annotations

import json

from requests import HTTPError


class ApiResourceError(HTTPError):
    def __str__(self) -> str:
        return f"ApiResourceError({self.args[0]}. Details: {self.details})"

    @property
    def details(self) -> str:
        try:
            return json.dumps(self.response.json())
        except json.JSONDecodeError:
            return "N/A"

    @classmethod
    def from_http_error(cls, http_error: HTTPError) -> ApiResourceError:
        return cls(http_error.args[0], response=http_error.response)



import os

from vellum.api_resources import Generate
from vellum.api_resources.completion_actuals import SubmitCompletionActuals
from vellum.api_resources.search import Search
from vellum.api_resources.types import (
    CompletionActual,
    DocumentUploadResult,
    GenerateRequest,
    ResultMerging,
    SearchFilters,
    SearchOptions,
    SearchWeights,
)
from vellum.api_resources.upload_document import UploadDocument

api_key = os.environ.get("VELLUM_API_KEY")

__all__ = [
    "CompletionActual",
    "Generate",
    "GenerateRequest",
    "ResultMerging",
    "Search",
    "SearchFilters",
    "SearchOptions",
    "SearchWeights",
    "SubmitCompletionActuals",
    "UploadDocument",
    "DocumentUploadResult",
    "api_key",
]



from setuptools import find_packages, setup

VERSION = "0.0.12"

setup(
    name="vellum-client",
    version=VERSION,
    description="Vellum Python client",
    url="http://github.com/vocify/vellum-client-python",
    author="vellum.ai",
    author_email="devops@vellum.ai",
    packages=find_packages(where="."),
    package_data={
        "vellum": ["py.typed"],
    },
    install_requires=[
        "dacite>=1.8.0",
        "requests>=2.20.0",
    ],
)
